unit PasKeyMatcher;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

var
  Identifiers: array[char] of Boolean;
  IsLetterChar: array[char] of Boolean;
  CharValTable: array[char] of Byte;
  HashValTable: array[Char] of Byte;
  PasKeyMatch: Array[0..13,0..25,0..25] of Boolean;
  KeywordsList: TStringList;

const
  RESERVED_WORDS_TP: array [1..54] of String = (
    'absolute', 'and', 'array', 'asm',
    'begin',
    'case', 'const', 'constructor',
    'destructor', 'div', 'do', 'downto',
    'else', 'end',
    'file', 'for', 'function',
    'goto',
    'if', 'implementation', 'in', 'inherited', 'inline', 'interface',
    'label',
    'mod',
    'nil', 'not',
    'object', 'of', 'on', 'operator', 'or',
    'packed', 'procedure', 'program',
    'record', 'reintroduce', 'repeat',
    'self', 'set', 'shl', 'shr', 'string',
    'then', 'to', 'type',
    'unit', 'until', 'uses',
    'var',
    'while', 'with',
    'xor'
  );

  RESERVED_WORDS_DELPHI: array [1..15] of String = (
    'as',
    'class',
    'except', 'exports',
    'finalization', 'finally',
    'initialization', 'is',
    'library',
    'on', 'out',
    'property',
    'raise',
    'threadvar',
    'try'
  );

  RESERVED_WORDS_FPC: array [1..5] of String = (
    'dispose', 'exit', 'false', 'new', 'true'
  );


function IsKeyWord(const s: string): Boolean;

type

TPascalCompilerMode = (
  pcmObjFPC,
  pcmDelphi,
  pcmFPC,
  pcmTP,
  pcmGPC,
  pcmMacPas
  );

TSynPasDividerDrawLocation = (
    pddlUnitSection,
    pddlUses,
    pddlVarGlobal,     // Var, Type, Const in global scope
    pddlVarLocal,      // Var, Type, Const in local (procedure) scope
    pddlStructGlobal,  // Class, Object, Record in global type block
    pddlStructLocal,   // Class, Object, Record in local (procedure) type block
    pddlProcedure,
    pddlBeginEnd,      // Includes Repeat
    pddlTry
  );



implementation

procedure InitKeywordList;
var
  i: integer;
begin
  if KeywordsList = nil then begin
    KeywordsList := TStringList.Create;
    for i := 1 to High(RESERVED_WORDS_TP) do
      KeywordsList.AddObject(RESERVED_WORDS_TP[i], TObject(pcmTP));
    for i := 1 to High(RESERVED_WORDS_DELPHI) do
      KeywordsList.AddObject(RESERVED_WORDS_DELPHI[i], TObject(pcmDelphi));
    for i := 1 to High(RESERVED_WORDS_FPC) do
      KeywordsList.AddObject(RESERVED_WORDS_FPC[i], TObject(pcmFPC));
  end;
  KeywordsList.Sorted := true;
end;

procedure InitKeywords(const s: string);
var I, Len: integer;
begin
  Len:= Length(S);
  for I:= 1 to Len do
  begin
    PasKeyMatch[I-1][CharValTable[S[I div 2]]][CharValTable[S[I]]]:= True;
  end;
end;

procedure InitTables;
var
  I, J: Char;
  K: integer;
begin
  for I := #0 to #255 do
  begin
    case I of
      '_', '0'..'9', 'a'..'z', 'A'..'Z': Identifiers[I] := True;
    else Identifiers[I] := False;
    end;
    J := UpCase(I);
    case I of
      'a'..'z', 'A'..'Z': CharValTable[I] := Ord(J) - 65;
    else
      CharValTable[Char(I)] := 0;
    end;
    IsLetterChar[I]:=(I in ['a'..'z','A'..'Z']);
  end;
  for K:= 0 to KeywordsList.Count -1 do  InitKeywords(KeywordsList[K]);
end;

function IsKeyWord(const s: string): Boolean;
var I, Len: integer;
begin
  Result:= False;
  Len:= Length(s);
  if (Len < 2) or (Len > 14) then exit;
  I:= 1;
  while I <  Len do
  begin
    if (not PasKeyMatch[I-1][CharValTable[S[I div 2]]][CharValTable[S[I]]]) then break;
    inc(I);
  end;
  Result:= I = Len;
end;

Initialization
InitKeywordList;
InitTables;
finalization
  FreeAndNil(KeywordsList);

end.

